//*****************************************************************************
//
//	Coordinator.m  
//		
//		NXApp delegate, central control object for GateKeeper 
// 
//			by	Felipe A. Rodriguez		
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//*****************************************************************************

#import <appkit/nextstd.h>
#import <appkit/Application.h>
#import <objc/NXStringTable.h>
#import <defaults/defaults.h>
#import <bsd/c.h>
#import <sys/dir.h>  /* POSIX applications #include <dirent.h> */

#import "GKdefs.h"
#import "Animator.h"
#import "Parse.h"
#import "OptionsEditor.h"
#import "options.h"
#import "IconView.h"
#import "ToolBar.h"
#import "DialServer.h"
#import "Timer.h"
#import "IdleAnimator.h"
#import "Coordinator.h"
#import "CommandScroll.h"
#import "GateDocEditor.h"
#import "InfoAnimator.h"
#import "DLDelegate.h"
#import "pppstats.h"




@interface Coordinator (Private)


		// RPC routines 
#import "pppd_rpc_types.h"
#import "pppd_rpc.h"
extern void gk_listen(void);					
extern void gk_check_out(void); 


@end
@implementation Coordinator (Private)
@end
@implementation Coordinator

//*****************************************************************************
//
//		state machine which monitors ppp daemon
//
//*****************************************************************************
 
- setPhase:(int)phase
{
static int linkStg;						// state of ppp link

	switch(phase)
		{							
		case 0: 						// pppd hangup
			if(linkStg != 1)
				{
			[theTimer stopTimer];
			[[theAnimator setOpenGate:YES] setStandbyGate:YES];
			[theAnimator startAnimTimer];				// cycle thru animation
											// disable the disconnect button
			[[[NXApp mainMenu] findCellWithTag:1] setEnabled:NO];
			[statusButton setEnabled:NO];
			[theOpenButton setEnabled:YES];		// enable the open doc button
			[dial setEnabled:YES];					// enable dial button
			[connectionSpeedField setStringValue:"                          "];
			[timeField setStringValue:"        "];
			[timeCell setStringValue:"      "];
			linkStg = 1;
			[commandView appendString:[self localString:"pppdn"]];
			[statusView setStringValue:[self localString:"pppdn"]];
				}
			break;
		case 2: 						// initial state after GK launch
			if(linkStg < 2)
				{
				[theTimer stopTimer];
				[commandView appendString:[self localString:"ppp is ready\n"]];
			[statusView setStringValue:[self localString:"ppp is ready\n"]];
			[statusButton setEnabled:NO];
				[connectionSpeedField setStringValue:"                      "];
				[timeField setStringValue:"        "];
				[timeCell setStringValue:"      "];
				[[[NXApp mainMenu] findCellWithTag:1] setEnabled:NO];
				[[[NXApp mainMenu] findCellWithTag:2] setEnabled:YES];
				linkStg = 2;
				}
			break;

		case 13:						// Connect script failed	
			[[theAnimator setOpenGate:YES] setStandbyGate:YES];
			[theAnimator startAnimTimer];				// cycle thru animation
			[commandView appendString:
								[self localString:"connect script failed\n"]];
			[statusView setStringValue:
								[self localString:"connect script failed\n"]];
			linkStg = 2;

		case 12: 						// dialing?
		case 3: 						
			if(linkStg < 3)
				{
				wasHidden =	[NXApp isHidden];					// app hidden?
				[[theAnimator setStandbyGate:NO] startAnimTimer];
				[dial setEnabled:NO];			// disable dial button
				[theOpenButton setEnabled:NO];	// disable the open doc button
				[commandView appendString:[self localString:"Connecting..."]];
				[commandView appendString:"\n"];
				[statusView setStringValue:[self localString:"Connecting..."]];
				[statusView setStringValue:[self localString:"Connecting..."]];
			if(strcmp("YES",NXGetDefaultValue([NXApp appName], DISPLAYS)) == 0)
					[[statusView window] makeKeyAndOrderFront:self];			
					// if auto unhide switch is set wait msec before hiding	
	if(strcmp(NXGetDefaultValue([NXApp appName], "autoUnhide"),"YES") == 0)	
					[NXApp perform:@selector(unhide:) with:self 
												afterDelay:2000 
												cancelPrevious:YES];				
				linkStg = 3;
				}
			break;

		case 6:							// PPP link is up
			if(linkStg < 6)
				{								// enable disconnect button
				[[[NXApp mainMenu] findCellWithTag:1] setEnabled:YES];
				[[[NXApp mainMenu] findCellWithTag:2] setEnabled:NO];
				[commandView appendString:[self localString:"pppup"]];
				[commandView appendString:"\n"];
				[self gotIt];
				linkStg = 6;
				}
			break;

		case 20:						// Inactivity timeout					
			[commandView appendString:
						[self localString:"ppp idle timeout is imminent\n"]];
			[statusView setStringValue:
						[self localString:"ppp idle timeout is imminent\n"]];
			idleZone = NXCreateZone(vm_page_size, vm_page_size, YES);
			[NXApp loadNibSection:"IdlePanel.nib" 
							owner:self 
							withNames:NO 
							fromZone:idleZone];
			[iTimer startAnimTimer];
			iTimer = nil;
			break;

		case 21:						// pppd died 	
			[[theAnimator setOpenGate:YES] setStandbyGate:YES];
			[theAnimator startAnimTimer];				// cycle thru animation
			[[[NXApp mainMenu] findCellWithTag:1] setEnabled:NO];
			[[[NXApp mainMenu] findCellWithTag:2] setEnabled:YES];
			[statusButton setEnabled:NO];
			[commandView appendString:[self localString:"ppp daemon died\n"]];
			[statusView setStringValue:[self localString:"ppp daemon died\n"]];
			linkStg = 0;
			break;

		default:	 		
			break;
		}
	NXPing();
		
    return self;
}
//*****************************************************************************
//
// 		return our string table (contains key->value localization pairs) 
//
//*****************************************************************************

- stringTable;
{
    return stringTable;
}
//*****************************************************************************
//
// 	localize error messages using stringTable as appropriate, displays
// 	the message in an alert panel
//
//*****************************************************************************

- showAlert:(const char *)errorString
{
	NXRunAlertPanel(0, [self localString:errorString], 
							[stringTable valueForStringKey:"OK"], NULL, NULL);

    return self;
}
//*****************************************************************************
//
// 	localize string messages using stringTable as appropriate
//
//*****************************************************************************

- (const char *)localString:(const char *)aString
{
const char *returnedString;
    
    if (returnedString = [stringTable valueForStringKey:aString])
    	return returnedString;

    return aString;
}
//*****************************************************************************
//
// 		Called when link is established
//
//*****************************************************************************
 
- gotIt
{
	[theAnimator removeTimedEntry];
													// play ppp up snd
	if(strcmp(NXGetDefaultValue([NXApp appName],"sound"), "YES") == 0) 
		[[[Sound findSoundFor: "majestic"] play: nil] free];

	[theTimer Fire:self];							// session timer start cnt
	[self pppstats:commandView];

	if(!onImage)									// display ppp up app icon
		onImage = [NXImage findImageNamed:"g4"];	
	[theIconView setImage:onImage];	
				
	[statusButton setEnabled:YES];
	[statusView setStringValue:[self localString:"pppup"]];

						// if auto hide switch is set wait 5 sec before hiding	
	if((strcmp(NXGetDefaultValue([NXApp appName],"autoHide"),"YES") == 0) &&
																	 wasHidden)	
		[NXApp perform:@selector(hide:) with:self 
								  afterDelay:3000 
							  cancelPrevious:YES];				

    return self;
}
//*****************************************************************************
//
// 		registers connection speed 
//
//*****************************************************************************

- connectedAt:(const char *)speed  
{
	strncpy(Path, [stringTable valueForStringKey:"connectAt"], MAXPATHLEN);
	strncat(Path, speed, MAXPATHLEN - strlen(Path));
	[connectionSpeedField setStringValue:Path];
	[commandView appendString:Path];
	[commandView appendString:"\n"];

    return self;
}
//*****************************************************************************
//
// 		displays captured info regarding current ppp session
//
//*****************************************************************************
 
- displayLinkStatus
{
	if(capturedStr[0] != NULL)				// in case we man dialed
		{									// we will not know connect speed
		[commandView appendString:"\n#################################\n"];
		[commandView appendString:"####\n"];
		[commandView appendString:"####  "];
		[commandView appendString:capturedStr[0]];
		}
	else
		[commandView appendString:"\n#################################"];
	[commandView appendString:"\n####\n"];
	[commandView appendString:[stringTable valueForStringKey:"localIP"]];
	[commandView appendString:capturedStr[1]];
//	[localIPField setStringValue:capturedStr[1]];
	[commandView appendString:"\n####\n"];
	[commandView appendString:[stringTable valueForStringKey:"remoteIP"]];
	[commandView appendString:capturedStr[2]];
	[commandView appendString:"\n####"];
//	[remoteIPField setStringValue:capturedStr[2]];
	[commandView appendString:"\n#################################\n"];

    return self;
}
//*********************** Application Object Delegation ***********************
//*****************************************************************************
//
// 			setup defaults data base cache before app is init'd
//
//*****************************************************************************

- appWillInit:sender
{
    static NXDefaultsVector myDefaults = {		// setup defaults database
        {DISPLAYD, "YES"},						// display diagnostics window
		{DISPLAYS, "YES"},						// display status window
		{"autoHide", "YES"},					// auto hide upon link		
		{"autoUnhide", "YES"},					// auto appear while linking		
		{"autoRedial", "YES"},			
		{"PersistCon", "YES"},			
		{"Persist", "down"},					// persistent connect trigger
		{"DispPPP", "YES"},						// display pppstats
		{DISPLAYT, "YES"},						// display toolbar
		{"DoD", "NO"},							// dial on demand
		{"sound", "YES"},						// play sound 
		{AITIMER, "YES"},						// icon displayed online time
		{"iTimeout", "Off"},					// inactivity unlink's us
		{"iTimeThreshold", "0"},				// inactivity threshold 
		{SELCELL, "0"},							// cell selected in hotList
		{FIRSTTIME, "YES"},						// first time thru?
		{"options", "/etc/ppp/options"},			
		{"ip-down", "/etc/ppp/ip-down"},			
		{"ip-up", "/etc/ppp/ip-up"},			
		{"resolv", "/etc/resolv.conf"},			
		{"syslog", "/etc/syslog.conf"},			
		{"messages", "/usr/adm/messages"},			
		{"remote", "/etc/remote"},			
		{"locFIFO", "/usr/adm/ppp2.2.debug"},				// pref form start
		{HLLIST, "/"},			
		{"BaudStr", "CARRIER"},								// pref form end
		{DIALINIT, "ATZ&D0L3"},			// modem init str for manual Dial
		{"dialPrefix", "ATD"},			// modem AT command prefix for dialing
		{MODEMPORT, "cufa"},			// modem port, used in releasing locks
		{"path", "/"},							// path for .Gate documents
		{"currentDoc", " "},					// current .Gate document
		{"lastNumDialed", "8675309"},				// default man dial number
		{"savedTime", "000000000000"},			// save time used this mo
		{"monthTime", "000000000000"},			// save mo,yr of this ses'n
		{"preMonthTime", "000000000000"},		// save previous mo,yr 
        {NULL}};									// make local cache of ddb
    NXRegisterDefaults([NXApp appName], myDefaults);	
		
    return self;
}
//*****************************************************************************
//
// 			standard init
//
//*****************************************************************************

- appDidInit:sender
{												
kern_return_t 	ret;							// MiG return value variable
port_t			server;							// port id
static int c;
static char buf[256];

	theAnimator = [[Animator alloc] init];		// get appIcon window
	theIconView = [theAnimator iconView];
	timeCell = [theIconView getTextCell]; 		// ret the iconView's textCell

				// load console window and have it remember its size and loc
	[NXApp loadNibSection:"Console.nib" owner:self withNames:NO];
	[[commandView window] setFrameUsingName:[[commandView window] title]];	
	[[commandView window] setFrameAutosaveName:[[commandView window] title]];
	if(strcmp(NXGetDefaultValue([NXApp appName], DISPLAYD),"YES") == 0)
		[[commandView window] makeKeyAndOrderFront:self];	// show console win
	
	if(strcmp("YES", NXGetDefaultValue([NXApp appName], DISPLAYT)) == 0)	
		[self toolBar:self];							// show toolbar if set

	if(strcmp("YES", NXGetDefaultValue([NXApp appName], DISPLAYS)) == 0)
		[[statusView window] makeKeyAndOrderFront:self];  // show status panel

    if(strcmp(NXGetDefaultValue([NXApp appName], FIRSTTIME),"YES") == 0) 
		{							// if first time this version was launched
		[self showInfo:self];	
		if(!NXWriteDefault([NXApp appName], FIRSTTIME, "NO"))
			NXRunAlertPanel(0,
					[stringTable valueForStringKey:"ddbWriteError"],
                	[stringTable valueForStringKey:"OK"],
                	NULL,
                	NULL);
		}

	if(strcmp("YES", NXGetDefaultValue([NXApp appName], AITIMER)) == 0)	
		appIconTime = YES;					// show online time in app icon

	theTimer = [[Timer allocFromZone:[self zone]] init];

	gk_listen();							// create listener thread

											// if dock autolaunched at startup
	if(strcmp("YES", NXGetDefaultValue([NXApp appName], "NXAutoLaunch")) == 0)	
		{
					// if auto hide switch is set wait x sec before hiding	
		if((strcmp(NXGetDefaultValue([NXApp appName],"autoHide"),"YES")==0))	
			[NXApp perform:@selector(hide:) with:self 
									afterDelay:100 
									cancelPrevious:YES];	
		}

	usleep(300);
	server = pppd_rpc_server_look_up();					// find pppd
	if (server != PORT_NULL)							// pppd is active
		{
		if ((ret = pppd_status(server, &c)) != KERN_SUCCESS) 			 
			[self showAlert:"Error getting ppp daemon status."];
		else
			{
			sprintf(buf,"pppd returned status:  %d \n",(int) c);
			[commandView appendString:buf];
			[self setPhase:c];
			}
		}
	else
		{							
   			// If we weren't asked to open any documents at launch time, then 	
			// we were launched by double clicking on the application instead 
			// of a document. If so check whether we should auto-launch w/def's
		if(!NXGetDefaultValue([NXApp appName], "NXOpen") &&
			!NXGetDefaultValue([NXApp appName], "NXOpenTemp") &&
				!NXGetDefaultValue([NXApp appName], "NXServiceLaunch"))
			{							// if pppd does not respond launch it
			[hotListDelegate link];		// with preferences selected doc 
			}
		}
					 
    return self;
}
//*****************************************************************************
//
// 		This method is performed whenever a user double-clicks on an icon in
// 		the Workspace Manager representing a Gate program document.  
// 		
// 		Brings up the gate doc editor which can edit or link using the doc
//
//*****************************************************************************

- (int)app:sender openFile:(const char *)path type:(const char *)type 
{
	[theGateDocEditor editGateDoc:path];
				
	return 1;
}
//*****************************************************************************
//
// 		invoked immediately after app is hidden and unhidden respectively
//
//			the following two methods are implemented to produce
//			normal main window/menu app behavior when toolBar is open
//
//******************************************************************************
- appDidHide:sender
{
	[[NXApp mainMenu] close];		// close the main menu  
									
    return self;
}

- appDidUnhide:sender
{
	[[NXApp mainMenu] makeKeyAndOrderFront:self];		 

    return self;
}
//*****************************************************************************
//
// 		Instantantiates the subprocess object which exec's pppd
//
//*****************************************************************************

- linkWithFile:(const char *)path  
{
static char commandLine[MAXPATHLEN + 1], buf[8]; 
int timeout;
id image;

	if([self runScript:"preLink"])				// exec pre link script
		{ 
												// build command line
	strncpy(commandLine, [[NXBundle mainBundle] directory], MAXPATHLEN);	 
	strncat(commandLine, "/pppd", MAXPATHLEN - strlen(commandLine));
												// idle timeout is set
	if(strcmp("Off", NXGetDefaultValue([NXApp appName], "iTimeout")) != 0)
		{	
		strncat(commandLine, " idle ", MAXPATHLEN - strlen(commandLine));
		timeout = atoi(NXGetDefaultValue([NXApp appName], "iTimeout")) * 60;
		sprintf(buf,"%d",(int)timeout);
		strncat(commandLine, buf, MAXPATHLEN - strlen(commandLine));
		}

	if(path == NULL)
		{				
		lastCall[0] = '\0';	
		if([hotListDelegate selGateDocOptionsPath] == NULL)
			commandLine[0] = '\0';
		else									// if hotlist points to valid
			{
			strncat(commandLine, " ipcp-accept-local ipcp-accept-remote", 
											MAXPATHLEN - strlen(commandLine));
			strncat(commandLine, " file ", MAXPATHLEN - strlen(commandLine));
			strncat(commandLine, [hotListDelegate selGateDocOptionsPath], 
											MAXPATHLEN - strlen(commandLine));
			strncpy(BPath,[hotListDelegate selGateDocOptionsPath], MAXPATHLEN);			
			[providerField setStringValue:[self localString:"manDial"]];
			}
		}
	else
		{
		strncat(commandLine, 
				" demand :192.42.172.4 ipcp-accept-local ipcp-accept-remote", 
				MAXPATHLEN - strlen(commandLine));

		strcpy(lastCall, path);			// stores path of last invocation
										// setup status panel
		[statusView setStringValue:[self localString:"connecting"]];
		[providerField setStringValue:[hotListDelegate extractName:path]];	
		strncpy(Path, path, MAXPATHLEN);
		strncat(Path, "/Icon.tiff", MAXPATHLEN - strlen(Path));
		if(!(image = [[NXImage alloc] initFromFile:Path]))		// if imge
			image = [NXImage findImageNamed: ".dir"];		// else use blank
		[statusImageView setImage:image];

		if((strlen([[NXBundle mainBundle] directory]) + 45 + 
					(2 * strlen(path)) + strlen(commandLine)) < MAXPATHLEN)	
			{									// if we won't overrun buffer
			strcat(commandLine, " file ");
			strcat(commandLine, path);
			strcat(commandLine, OPTION);	// options file name wrapper
			strcat(commandLine, " connect \"");				
			strcat(commandLine, [[NXBundle mainBundle] directory]);
			strcat(commandLine, "/chat -v -f ");
			strcat(commandLine, path);
			strcat(commandLine, PPPUP);	// pppup file name wrapper
			strcat(commandLine, "\"");
			}
		else
			perror("command line is longer than MAXPATHLEN");
		strncpy(BPath, path , MAXPATHLEN);	
		strncat(BPath, OPTION, MAXPATHLEN - strlen(BPath));	// options 
		}
	if(commandLine[0] != '\0')				// if we have a valid commline 
		{
		if([self debugFlag:BPath])
			{				// if debug sw set in options disg commandline
			[commandView appendString:commandLine];
			[commandView appendString:"\n\r"];
			}
		system(commandLine);
		}
	else
		[self showAlert:"GateKeeper error building ppp command line."];
		}
		
	return self;
}
//*****************************************************************************
//
// 		Use terminal to do a manual link 
//
//*****************************************************************************

- Dial:sender  
{								
	if([self runScript:"preLink"])
		{ 
		manDial = YES;
		[self controlDaemon:self];			// kill the daemon
		usleep(800);
		[self setPhase:3];

		if(!theGateServer)					// create distributed Obj server
			theGateServer = [[DialServer alloc] init];  		
		GateConnection = [NXConnection registerRoot: theGateServer
						withName:"GateKeeperDialServer"];// DO name of server
		[GateConnection runFromAppKit];	// listen for DO messages from appKit
		if(![[NXBundle mainBundle]getPath:Path forResource:"MODEM"ofType:NULL])
			[self showAlert:"Error getting path for MODEM file"];
		[[Application workspace] openFile:Path withApplication:"Terminal"];	
		}

    return self;
}
//*****************************************************************************
//
// 		Called when menu item Unlink is pressed.  
//
//*****************************************************************************

- disconnect:sender  
{
kern_return_t 	ret;							// MiG return value variable
port_t			server;							// port id

	if([self runScript:"preUnLink"])
		{ 
		[commandView appendString:
							[self localString:"terminating ppp session\n"]];
		[statusView setStringValue:
							[self localString:"terminating ppp session\n"]];
		server = pppd_rpc_server_look_up();			// find pppd
		if (server == PORT_NULL)
			[self showAlert:"unable to find pppd server"];
		else
			{
			ret = set_pppd(server, 7);					// set pppd state
			if (ret != KERN_SUCCESS) 
				[self showAlert:"Call to pppd server failed."];
			}
		if(manDial)
			[self controlDaemon:self];	
		manDial = NO;
		}

    return self;
}
//*****************************************************************************
//
// 		run the preLink or preUnLink scripts, proceed if exit status true 
//
//*****************************************************************************

- runScript:(const char *)type  
{
id exitStatus = self;
int fd;

	strncpy(Path, [[NXBundle mainBundle] directory], MAXPATHLEN);
	strncat(Path, "/", MAXPATHLEN - strlen(Path));
	strncat(Path, type, MAXPATHLEN - strlen(Path));
	if(fd = open(Path, O_RDONLY) != -1)			//	if script exists
		{
		close(fd);
		if(system(Path) != 0)					// exec script, test exit
			exitStatus = nil;
		strcpy(Path, type);
		if(!exitStatus)
			strcat(Path," script exit status prevents requested action\n");
		else
			strcat(Path, " script executed sucessfully\n");
		[commandView appendStringUseFixedFont:Path];
		}

    return exitStatus;
}
//*****************************************************************************
//
// 		we can always edit another gate doc
//
//*****************************************************************************

- (BOOL)appAcceptsAnotherFile:sender 
{
    return YES;
}
//*****************************************************************************
//
//		termination is imminent 
//
//*****************************************************************************

- appWillTerminate:sender
{
	if(NXRunAlertPanel(0, [self localString:"Terminate ppp daemon?"],
							[stringTable valueForStringKey:"Yes"],
							[stringTable valueForStringKey:"No"],
							NULL) == NX_OKTAG)
		{
		[self killDaemon];
		[self runScript:"postPPP"];			// to do after daemon is killed
		}

	gk_check_out();			// remove our name from net name server

	[theAnimator removeTimedEntry];
	if(theGateServer)					
		[theGateServer appWillTerminate];
	if(hotListDelegate)					
		[hotListDelegate appWillTerminate];
	
    return self;
}
//*****************************************************************************
//
// 		kill pppd
//
//*****************************************************************************

- killDaemon
{
port_t server;									// port id
FILE *ff; 
int pid;

	server = pppd_rpc_server_look_up();			// find pppd
	if (server == PORT_NULL)
		fprintf(stderr,"lookup of pppd server failed in (killDaemon).\n");
	else
		die_pppd(server, 8);					// async RPC call to pppd

	usleep(900);
									// if the lock file is still around kill 
									// the daemon with extreme prejudice
    if((ff = fopen( "/etc/ppp/ppp0.pid", "r")) != NULL)
		{			 
		fprintf(stderr,"Killing ppp daemon with extreme prejudice.\n");
    	if( fscanf( ff, "%d", &pid) < 1) 
        	perror("Unable to read pid from ppp0.pid\n");
		else
			{
    		fclose( ff);
    		if( kill( pid, SIGKILL) == -1) 
				fprintf(stderr,"Error killing ppp daemon in (killDaemon).\n");
			}
    	}
	
    return self;
}
//*****************************************************************************
//
//		start and stop the ppp daemon 
//
//*****************************************************************************

- controlDaemon:sender 
{
	if(strcmp([cDaemonButton title],[self localString:"Stop pppd "]) == 0)
		{		
		[self setPhase:0];
		[commandView appendString:[self localString:"Killing ppp daemon\n"]];
		[self killDaemon];
		[cDaemonButton setTitle:[self localString:"Start pppd"]];
		}
 	else
		{
		[commandView appendString:[self localString:"Starting ppp daemon\n"]];
						// preferences selected HotList doc for launching pppd
		[hotListDelegate link];	
		[cDaemonButton setTitle:[self localString:"Stop pppd "]];
		}

	return self;
}
//*****************************************************************************
//
// 		view the ppp log file 
//
//*****************************************************************************

- viewLog:sender
{														// if readable
	if([self readable:NXGetDefaultValue([NXApp appName],"locFIFO")])
		[[Application workspace] 						// call edit to open
						openFile:NXGetDefaultValue([NXApp appName],"locFIFO") 
						withApplication:"Edit"];
	
    return self;
}
//*****************************************************************************
//
// 		show the info panel 
//
//*****************************************************************************

- showInfo:sender
{
    if(!infoPanel)
		[NXApp loadNibSection:"InfoPanel.nib" owner:self withNames:NO];
    [infoPanel startAnimTimer];
	
    return self;
}
//*****************************************************************************
//
//		show preferences panel   
//
//*****************************************************************************
 
- preferences:sender
{
	if(!preferencesPanel) 
		{
		[self loadBundlesOfType:"prefBundle" owner:NXApp nib:NO];
		preferencesPanel = [NXApp loadNibSection:"Preferences.nib" owner:NXApp 
															   withNames:YES];
		}
	[preferencesPanel makeKeyAndOrderFront:self];
		
    return self;
}
//*****************************************************************************
//
//		show NXHelpPanel 
//
//*****************************************************************************

- showHelpPanel:sender 
{
    if(!helpPanel)
		helpPanel = [NXHelpPanel new];
    [helpPanel display];
    [helpPanel makeKeyAndOrderFront:self];
	
    return self;
}
//*****************************************************************************
//
//		display the Status panel 
//
//*****************************************************************************

- showStatusPanel:sender 
{
	if(!NXWriteDefault([NXApp appName], DISPLAYS, "YES"))
		[self showAlert:"ddbWriteError"];

 	return [[statusView window] makeKeyAndOrderFront:self];
}
//*****************************************************************************
//
// 		display the Console window 
//
//*****************************************************************************

- showConsole:sender
{
	if(!NXWriteDefault([NXApp appName], DISPLAYD, "YES"))
		[self showAlert:"ddbWriteError"];

	return [[commandView window] makeKeyAndOrderFront:self];
}
//*****************************************************************************
//
//		show the timer 
//
//*****************************************************************************

- showTimerPanel:sender 
{
    return [theTimer showTimerPanel:self];
}
//*****************************************************************************
//
// 		called by timer in order to pass us appIcon time string 
//
//		also used in determining inactivity timeout (since this is called
//		once per minute during ppp sessions).
//
//*****************************************************************************

- showMenuTimer:(char *)buffer 
{
	if(appIconTime)
		[timeCell setStringValue:buffer];
	[timeField setStringValue:buffer];
	[theIconView setImage:onImage];
	[self pppstats];

	return self; 
}
//*****************************************************************************
//
// 		enable/disable display of online time in app icon 
//
//*****************************************************************************

- setAppIconTimer:(BOOL)onOff 
{
	appIconTime = onOff;

	return self; 
}
//*****************************************************************************
//
// 		return a pointer to the document list delegate 
//
//*****************************************************************************

- hotListDelegate 
{
	return hotListDelegate; 
}
//*****************************************************************************
//
//		returns whether a Gate doc may be opened or edited by real user
//
//*****************************************************************************

- readable:(const char *)nameOfFile
{
FILE *fp;

	if ((fp = fopen(nameOfFile, "r+")) == NULL)
		[self showAlert:"Unable to open Gate doc"];
	else
		{
		if(readable(fileno(fp)))
			{
			fclose(fp);
  			return self;				// we have permission
			}
		[self showAlert:"Access to Gate doc denied"];
		fclose(fp);
		}

   	return nil;						// we do not have permission
}
//*****************************************************************************
//
// 		returns the Gate doc options editor
//
//*****************************************************************************

- optionsEditor
{
	if(!theOptionsEditor)
		theOptionsEditor = [[OptionsEditor alloc] init];

    return theOptionsEditor;
}
//*****************************************************************************
//
// 		returns the state of the debug flag in an options file
//
//*****************************************************************************

- (BOOL)debugFlag:(const char *)optionFile
{
  	return [[[self optionsEditor] parseOptions:optionFile] debug];
}
//*****************************************************************************
//
// 			show the Tool Bar
//
//*****************************************************************************

- toolBar:sender
{
	if(!NXWriteDefault([NXApp appName], DISPLAYT, "YES"))
		[self showAlert:"ddbWriteError"];
	if(!toolBar) 
		[NXApp loadNibSection:"ToolBar.nib" owner:self withNames:NO];
	[toolBar makeKeyAndOrderFront:self];

    return toolBar;
}
//*****************************************************************************
//
// 		return the app icons view
//
//*****************************************************************************
 
- appIconView
{
	return 	theIconView;					
}
//*****************************************************************************
//
// 		return commandview 
//
//*****************************************************************************

- commandView 
{
	return commandView; 
}
//*****************************************************************************
//
// 		called by Dial server when remote manual Dial Tool's ports are 
//		invalidated
//
//*****************************************************************************
 
- DOFinished
{
	[self setPhase:0];
	[self controlDaemon:self];			// restart the daemon
	manDial = NO;

    return self;
}
//*****************************************************************************
//
// 	free simply gets rid of everything we created
// 	This is how nice objects clean up.
//
//*****************************************************************************

- free
{
	[timeCell free];
	if(onImage)
		[onImage free];
	if(toolBar) 
		[toolBar free];
	if(iTimer) 
		[iTimer free];

    return [super free];
}
//*****************************************************************************
//
// 		search /etc/syslog.conf to find the location of the named pipe used 
//		in recieving the output from pppd/syslog 
//
//*****************************************************************************

- fifo
{
static char del1[] = {". =:|\t\r\n"}, del2[] = {" \t\r\n"};
char *fifoName = NULL;

	if(!Parser)									
		Parser = [[Parse alloc] init];			// create parser to find port
	[[Parser setKey1:"local2"] setKey2:"debug"];		// set search pattern  
	[[Parser setDelim1:del1] setDelim2:del2];			// and delimiters

	if(fifoName = [Parser parseFile:"/etc/syslog.conf"])
		{
		if(!NXWriteDefault([NXApp appName], "locFIFO", fifoName))
			NXRunAlertPanel(0,
					[stringTable valueForStringKey:"ddbWriteError"],
                	[stringTable valueForStringKey:"OK"],
                	NULL,
                	NULL);
		free(fifoName);
		}
	else
		[self showAlert:"FIFO Path Error"];

    return self;
}
//*****************************************************************************
//
// 		checks the mail queue for mail awaiting delivery 
//
//*****************************************************************************

- (BOOL)mailInQueue 
{
int	eCntr = 0;
struct direct *dirp;
DIR *dp;
BOOL mail = NO;

    if ((dp = opendir("/usr/spool/mqueue")) != NULL)
		{
		while ((dirp = readdir(dp)) && eCntr < 3) 	// read dir and cnt entries
			{
			if(*dirp->d_name != '.')				// don't count if sys file
				eCntr++;
			}
		if(eCntr > 2)
			mail = YES;
		closedir(dp);
		}
	else
		[self showAlert:"Error opening Mail queue directory"];

	return mail;
}
//*****************************************************************************
//
// 		search app wrapper and load bundles of the specified type 
//
//*****************************************************************************

- loadBundlesOfType:(const char *)bundleType owner:o nib:(BOOL)loadNib
{
struct direct *dirp;
DIR *dp = NULL;
char *ptr, *p;

    if ((DIR *)(dp = opendir([[NXBundle mainBundle] directory])) != NULL)
		{										// open DIR stream
		while (dirp = readdir(dp)) 				// read dir and find all
			{									// entries with ext .bundle
			if(dirp->d_namlen > strlen(bundleType))
				{								
				ptr = dirp->d_name;
													// str end - strlen of ext
				ptr += (dirp->d_namlen - strlen(bundleType));							
				if(strcmp(ptr, bundleType) == 0)	// if extension is of type
					{
					p = ptr = NXCopyStringBuffer(dirp->d_name);
													// str end - strlen of ext
					ptr += (dirp->d_namlen - strlen(bundleType) - 1);	
					*ptr = '\0';
					[self loadBundle:p ofType:bundleType owner:o nib:loadNib];
					free(p);
					}
				}
			};
		closedir(dp);
		}
	else
		[[NXApp delegate] showAlert:"Error opening mainbundle directory."];

	return self;							
}
//*****************************************************************************
//
// 		load a bundle of the specified type  
//
//*****************************************************************************

- loadBundle:(const char *)p ofType:(const char *)bType owner:o nib:(BOOL)lNib
{
NXBundle *myBundle = nil;

	if(![[NXBundle mainBundle] getPath:Path forResource:p ofType:bType])
		NXRunAlertPanel(0,"Error getting bundle path",0, 0, 0);
	else
		{
		if(myBundle = [[NXBundle alloc] initForDirectory:Path])
			{
			[myBundle principalClass];
			if(lNib)									// should we load a nib
				{
				if(![myBundle getPath:Path forResource:p ofType:"nib"])
					{
					NXRunAlertPanel(0,"Error getting nib path",0,0,0);
					NXRunAlertPanel(0, Path,0,0,0);
					}
				else
					[NXApp loadNibFile:Path owner:o withNames:YES 
													 fromZone:[self zone]];
				}
			}
		}

	return self;							
}
//*****************************************************************************
//
// 		enable pppstats, if aView is nil open w/o a view for stats only
//
//*****************************************************************************
 
- pppstats:aView
{
											// display pppstats?
	if(strcmp(NXGetDefaultValue([NXApp appName], "DispPPP"),"YES") == 0)
		{
		openSocket(aView);		// open IP datagram socket, pass a view
		intpr();
		pppstats = YES;
		}
	else
		pppstats = NO;	

	return self;
}
//*****************************************************************************
//
// 		poll pppstats
//
//*****************************************************************************
 
- pppstats
{
struct pppIO *pppStat;

	if(pppstats)
		pppStat = intpr();
		
	return self;
}
//************** Diagnostics Window delegate methods **************************
//*****************************************************************************
//
// 		called whenever the user minituriazes our Diagnostics window.
//
//*****************************************************************************

- windowWillMiniaturize:sender toMiniwindow:miniwindow 
{
    return [sender setMiniwindowIcon:"miniWinIcon"];
}
//*****************************************************************************
//
// 		called whenever the user closes our Diagnostics window.
//
//*****************************************************************************

- windowWillClose:sender  
{
	if(![sender isKindOf:[Panel class]])	// simple test to see if diag
		{									// win is what is being closed
		if(!NXWriteDefault([NXApp appName], DISPLAYD, "NO"))
			[self showAlert:"ddbWriteError"];
		}

    return self;
}

@end

