//*****************************************************************************
//
//	m.m.  
//
//		This is a Multi-tasked, Multi-threaded tool which acts as a frontend 
//		to a serial port, a rearend for Terminal.app and which avails itself  
//		to GateKeeper through distributed objects.
//
//			by	Felipe A. Rodriguez		
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//*****************************************************************************

#import "GateTool.h"
#import "m.h"
#import <appkit/nextstd.h> 
#import <dpsclient/dpsNeXT.h>
#import <soundkit/Sound.h>
#import <stdio.h>
#import <sys/types.h>
#import <sys/ioctl.h>
#import <signal.h>
#import <termios.h>
#import <objc/hashtable.h>
#import <remote/NXProxy.h>
#import <machkit/NXPort.h>

#define	MAXSTRPAIRS 60		// max num of expect/send str pairs in modemInit 
#define	SYSBEEP "/NextLibrary/Sounds/SystemBeep.snd"		 
#define	LOCK_PREFIX	"/usr/spool/uucp/LCK/LCK.."

	id GateKeeperProxy;					// proxy for GateKeeper server
	id DialServer;						// id of GateTool's DO server
	id GateConnection;					
	volatile int tty;					// controlling tty
	char Path[MAXPATHLEN + 1];			// generic buffer to hold paths
	volatile int loopcntr;				// counter for steps in script file
	int ScriptRunning = 0;				// On/Off for script mode 
	volatile int initMODEM = 1;			// are we sending a single AT command?
	volatile int pError = 0;			// set if error occurs and wish to exit
	char *tmpNames[MAXSTRPAIRS]; 		// Ptr's for script steps
	char *tmpTypes[MAXSTRPAIRS];
	mutex_t lock;	// assures completion of new thread before we continue
	static char *lock_file;
//	struct sgttyb setp = {B50,B50,(char)0x7f,(char)0x15,(CBREAK|ANYP|!ECHO)};
	struct sgttyb setp = {B50,B50,(char)0x7f,(char)0x15,(RAW|ANYP|!ECHO)};
		
#ifndef pid_t				// not declared in sys/types header where it s/b
	#define pid_t int
#endif	/* pid_t */

extern pid_t getppid(void);			

//int logfd = 0, logfd2 = 0;			// enable for diagnostics



//*****************************************************************************
//
// 		main 
//
//*****************************************************************************

int main(int argc, char *argv[]) 
{
struct stat statbuf;
static int Child, nread = 0;
static char buff[MAXPATHLEN];

	Init();											

    switch (Child = vfork())		// create new process without fully copying
   	 	{							// the address space of the old process
    	case -1:	// error
			perror("Error during vfork in MODEM task.");
			break;
			
    	case 0:  	// child -- Vfork returns 0 in the child's context
			for(;;)				// Child reads user shell and writes to Pty
				{
				if((nread = read(0, buff, MAXPATHLEN)) < 0)
					break;				//EOF on STDIN indicates we're done
				if(write(tty, buff, nread) != nread)
					perror("Error writing to Master Pty.");
				}
			kill(getppid(), SIGTERM);	//NOTIFY parent we have recieved EOF
			break;						// and are terminating
			
    	default:	// parent --  vfork returns pid of the child in the 			
					//	 			parent's context
			signal(SIGALRM, catchalarm);  // alarm in case expect str not ret'd
			if (stat(SYSBEEP, &statbuf) == 0)					
				SNDPlaySoundfile(SYSBEEP, NX_BASETHRESHOLD);		// beep
			sendScriptToModem("");
			for(;;)				// Parent reads Pty and writes to user shell
				{
				if((nread = read(tty, buff, MAXPATHLEN)) <= 0)
					{											// EOF or ERROR
					if([GateKeeperProxy shouldExit] || pError) 				
						break;									// user exit
					else							// else reopen serial port
						openSerialPort([GateKeeperProxy modemPort]);
					}
				if(write(1, buff, nread) != nread)
					perror("Error writing to stdout.");
				if(ScriptRunning)
					{buff[nread] = '\0'; sendScriptToModem(buff); }
				}
			kill(Child, SIGTERM);
			break;					// and are terminating
    	}
	childDidExit();
    exit(0);
}
//*****************************************************************************
//
// 		cleanup after a child process exits
//
//*****************************************************************************

void childDidExit(void)
{
//    if (childPid)
		{
				// stty  --  reset parameters of the users Terminal Shell  
//		(void)ioctl(tty, TIOCSETP, (char *)&orgp);
		close(tty);
//		fclose(fpToChild);
//		childPid=0;		// specify that child is dead
		}
		
    return;
}
//*****************************************************************************
//
// 		GateServer thread 
//
//		This thread avails GateToolServer, passes it the fd of the pty 
//		and then awaits a distributed objects connection to the server.
//
//*****************************************************************************

any_t gateThread(any_t arg)
{
	GateConnection = [NXConnection registerRoot: DialServer
        				withName:"GateToolServer"];	// DO name of server
	[DialServer setPty:0];						// pass fd for our stdin
	[NXPort worryAboutPortInvalidation];		// send invalidate when we end
	mutex_unlock(lock);							// remove mutex lock
	[GateConnection run];

	return arg;
}
//*****************************************************************************
//
// 		process send/expect str pairs to and from a MODEM.
//		
//		three keywords are used SEND (send, w/echo), EXPECT (expected reply)
//		and PASS (send, w/o echo)
//
//*****************************************************************************

void sendScriptToModem(char *buffer)
{
static char view[MAXPATHLEN + 1];
static char strbuf[MAXPATHLEN + 1];
static int chrcnt = 0, cntr = 0, strpos = 0;
static char minBuffer[2] = {"b"};

	alarm(4);											// turn on alarm
	while(*buffer && (strpos < MAXPATHLEN))				// build lines for
		{												// processing, in 
		strbuf[strpos++] = *buffer++; 					// case we recieve
		strbuf[strpos] = '\0';							// str fragments
		}
	if(strcmp("EXPECT", tmpNames[cntr]) == 0)
		{
		if(strstr(strbuf, tmpTypes[cntr]))
			{
			free(tmpTypes[cntr]);						// clean up as we go
			free(tmpNames[cntr]);					
			cntr++;
			strpos = 0;
			}
		}
	if((strcmp("SEND", tmpNames[cntr]) == 0) || 
						(strcmp("PASS", tmpNames[cntr]) == 0))
		{
		if(chrcnt == 0) 							// if sending a new command
			sprintf(view,"%s\r",tmpTypes[cntr]);
		if(strcmp(minBuffer, strbuf) == 0 || chrcnt == 0)
			{
			minBuffer[0] = view[chrcnt++];			// send one chr at a time
			if(write(tty, minBuffer, 1) != 1)		// write to open serial
				perror("Error writing to serial port.");	 				
			}											
		strpos = 0;
		if(view[chrcnt] == '\0')
			{
			chrcnt = 0;						// Reset chrcnt when view str sent 
			free(tmpTypes[cntr]);			// clean up as we go		
			free(tmpNames[cntr]);					
			cntr++;							// Increment Script step cntr	
			}
		else
			{ 
			if(view[0] == '\r') sendScriptToModem("\r");	// Echo to call
			if(strcmp("PASS", tmpNames[cntr]) == 0)			// ourselves if we
				sendScriptToModem(minBuffer);				// are sending nl	
			}												// or a password
		}
	if(cntr >= (loopcntr - 1))				// Have we sent entire script yet?
		{
		ScriptRunning = 0;					// turn off script read/write
		alarm(0);											
		cntr = 0;
		if(initMODEM)						// if we just init'd the MODEM
			{
			[GateKeeperProxy showDialPanel];
			initMODEM = 0;			// single command mode once modem is init'd
			}
		}		

    return;
}
//*****************************************************************************
//
// 		convert a spc delimited str into a script to be sent with func
//		sendScriptToModem(char *buffer)  		i.e. "PASS ~. "
//
//*****************************************************************************

void strToScript(const char *buffer)
{
static char *str, delim[] = {" "}, *argString;

	loopcntr = 0;									// reset loopcntr global
	argString = NXCopyStringBuffer(buffer);
	str = strtok(argString, delim);					// Parse string and test 
	while((str != NULL) && (loopcntr < (MAXSTRPAIRS/2)))									
		{											
		tmpNames[loopcntr] = NXCopyStringBuffer(str);
		str = strtok(NULL, delim);					// next token 
		if(str != NULL)
			{
			tmpTypes[loopcntr++] = NXCopyStringBuffer(str);
			str = strtok(NULL, delim);				// next token 
			}
		}
	ScriptRunning = 1;								// script mode is on
	if(!initMODEM)								// if we just init'd the MODEM
		loopcntr++;

    return;
}
//*****************************************************************************
//
// 		initialize ourselves to use this serial port
//
//*****************************************************************************

void Init(void)
{
const char *deviceName;

//	if(!logfd)										// enable for diagnostics
//		logfd = open("/GKlog", O_WRONLY|O_CREAT|O_TRUNC, 644);
//	write(logfd, "Init", strlen("Init"));

	initThreaded();									// setup DO in new thread
	strToScript([GateKeeperProxy modemInit]);		// init modem
	deviceName = [GateKeeperProxy modemPort];		// query GK for serial port
	
	setp.sg_ispeed = setp.sg_ospeed = 				// query GK for DTE speed
						translate_speed(atoi([GateKeeperProxy modemSpeed]));

	if((tty = open("/dev/tty", O_RDWR)) == -1)		// open controlling tty 
		perror("Error opening MODEM tool's controlling tty."); 
	else			// stty  --  set parameters of the users Terminal Shell 
		(void)ioctl(tty, TIOCSETP, (char *)&setp);

	lockPort(deviceName);						// lock serial port
	openSerialPort(deviceName);

	return;
}
//*****************************************************************************
//
// 		open the serial port
//
//*****************************************************************************

void openSerialPort(const char *deviceName)
{
	strncpy(Path, "/dev/", MAXPATHLEN);		// generate path to serial port
	strncat(Path, deviceName, MAXPATHLEN - strlen(Path));
	if((tty = open(Path, O_RDWR)) == -1)			// open the serial port
		{
		fprintf(stderr, "\n\rError opening serial port: %s   ", Path);
		if(strlen(deviceName) < 2)					// serial port is invalid
			{
			fprintf(stderr, "\n\rGateKeeper returned an invalid serial port.");
			fprintf(stderr, "\n\rCheck the selected document in GateKeeper's");
			fprintf(stderr, "\n\rpreferences for a valid serial port ");
			fprintf(stderr, "setting.\n\r\n\r");
			pError = 1;
			}
     	sleep(2);
		}
	else					// stty  --  set parameters of the serial port 
		(void)ioctl(tty, TIOCSETP, (char *)&setp);

	return;
}
//*****************************************************************************
//
// 		instantantiate the distributed objects server in a new thread
//		notify gatekeeper we are up
//
//*****************************************************************************

void initThreaded(void)
{
	lock = mutex_alloc();

 	DialServer = [[GateTool alloc] init];			// create new DialServer
	mutex_lock(lock);		// place mutex lock to be released in new thread
							// create and detach a new thread at func
	cthread_detach(cthread_fork((cthread_fn_t)gateThread, (any_t)0));
	mutex_lock(lock);		// place mutex lock, s/fail if new thread not oper
	GateKeeperProxy = [NXConnection connectToName:"GateKeeperDialServer"];
				// protocol tells DO sys this is a oneway call so that
	if(GateKeeperProxy)		// we do not generate an uncaught exception 
		[GateKeeperProxy setProtocolForProxy:@protocol(DialServer)];
	[GateKeeperProxy activateGateKeeper];
	mutex_unlock(lock);							// remove mutex lock

    return;
}
//*****************************************************************************
//
// 		called if alarm expires while sending commands to the modem or 
//		awaiting a response from it.  Also performs a normal exit.
//
//*****************************************************************************

void catchalarm(int a)
{
	unlockPort();		
	if(initMODEM == 0)						// if MODEM was initialized
		exit(0);
	else
		perror("Timeout error while initializing the modem");
	exit(-1);

    return;
}
//*****************************************************************************
//
// 		lock - create a lock file for the named lock device 
//
//*****************************************************************************

int lockPort(const char *dev)
{
int fd, pid, n;
char *p;

    if ((p = strrchr(dev, '/')) != NULL)
		dev = p + 1;
    lock_file = malloc(strlen(LOCK_PREFIX) + strlen(dev) + 1);
    if (lock_file == NULL)
		perror("lock file name missing");
    strcat(strcpy(lock_file, LOCK_PREFIX), dev);

    while ((fd = open(lock_file, O_EXCL | O_CREAT | O_RDWR, 0644)) < 0) 
		{
		if(errno == EEXIST && (fd = open(lock_file, O_RDONLY, 0)) >= 0) 
			{	/* Read the lock file to find out who has the device locked */
			n = read(fd, &pid, sizeof(pid));
			if (n <= 0) 
				{
			  fprintf(stderr,"Can't read pid from lock file %s\n\r",lock_file);
				close(fd);
				} 
			else 
				{
				if (kill(pid, 0) == -1 && errno == ESRCH) 
					{    /* pid no longer exists - remove the lock file */
					if (unlink(lock_file) == 0) 
						{
						close(fd);
						fprintf(stderr,"Removed stale lock on %s (pid %d)\n\r",
													dev, pid);
						continue;
						} 
					else
					   fprintf(stderr,"Can't remove stale lock on %s\n\r",dev);
					} 
				else
				   fprintf(stderr,"Device %s is locked by pid %d\n\r",dev,pid);
				}
			close(fd);
			} 
		else
			fprintf(stderr,"Can't create lock file %s: \n\r", lock_file);
		free(lock_file);
		lock_file = NULL;
		return -1;
		}
    pid = getpid();
    write(fd, &pid, sizeof pid);
    close(fd);

    return 0;
}
//*****************************************************************************
//
// 		 unlock - remove our lockfile 
//
//*****************************************************************************

void unlockPort()
{
    if (lock_file) 
		{
		unlink(lock_file);
		free(lock_file);
		lock_file = NULL;
    	}
}
//*****************************************************************************
//
// 		Translate from bits/second to a speed_t 'B'prefix
//
//*****************************************************************************

int translate_speed(int bps)
{
struct speed *speedp;

    if (bps == 0)
		return 0;
    for (speedp = speeds; speedp->speed_int; speedp++)
		if (bps == speedp->speed_int)
	    	return speedp->speed_val;
	fprintf(stderr, "speed %d not supported\n", bps);

    return 0;
}

