//*****************************************************************************
//
//	SelectPreference.m.  
//
//		Preferences logic.
//
//			by	Felipe A. Rodriguez		
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//*****************************************************************************

#import "../SwapView.h"
#import "../Coordinator.h"
#import "../GKdefs.h"
#import "../DLDelegate.h"
#import "SelectPreference.h"
#import "SelectPanelController.h"

#define BUTTONPOSITION 4				// position of this inspectors' button




@implementation SelectPreference 

//*****************************************************************************
//
// 		recieved immediately after class is dynamically loaded 
//
//*****************************************************************************

+ finishLoading:(struct mach_header *)header
{
	[self poseAs:[self superclass]];

	return self;
}
//*****************************************************************************
//
//		sent to us after nib objects are unarchived and init'd 
//
//*****************************************************************************
 
- awakeFromNib
{
NXRect fRect;								// scroll's frame rect
NXSize sSize;								// size of scroll's doc view
NXCoord newWidth;
NXImage *image = [NXImage findImageNamed:"app"];

	[super awakeFromNib];
								// get coordinates for selection button
	[prefScrollView getFrame:&fRect];
	[prefScrollView getContentSize:&sSize];
	fRect.origin.x = 0.0;
	fRect.origin.y = -10.0;
	fRect.size.width = 60.0;

								// move x origin right as buttons are added
	fRect.origin.x += fRect.size.width * BUTTONPOSITION;
								// if origin of new button plus its width
								// exceeds the scroll's docView width
								// resize the scroll's docView 
	newWidth = fRect.origin.x + fRect.size.width;
	if(newWidth > sSize.width)
		[[prefScrollView docView] sizeTo:newWidth :sSize.height];

								// find button image
	strncpy(Path, [[NXBundle mainBundle] directory] , MAXPATHLEN);	
	strncat(Path,"/Select.prefBundle/icon.tiff",MAXPATHLEN - strlen(Path));
	image = [[NXImage alloc] initFromFile:Path];

								// add a selector button for this inspector
	[bView addSubview:[[[[[[[Button alloc] 
						initFrame:&fRect] 
						setImage:image]
						setTag:BUTTONPOSITION]
						setType:NX_PUSHONPUSHOFF]
						setTarget:self]
						setAction:@selector(displaySelect:)]];
	[bView display];
	
    return self;
}
//*****************************************************************************
//
//		swap preferences subviews, dynamically load ours if necessary
//
//*****************************************************************************

- displaySelect:sender 
{
	[self loadPreference:"SelectPanelController" 
				 ofClass:[SelectPanelController class] 
				 button:sender];
				 
    return self;
}
//*****************************************************************************
//
// 		as text delegate we recieve this when textfield cell is being edited 
//
//*****************************************************************************

- (BOOL)textWillChange:textObject
{
	[[swapView window] setDocEdited:YES];	// indicate change is not saved

    return NO;
}
//*****************************************************************************
//
// 		text delegate receives when editing of textfield cell is ended 
//
//*****************************************************************************

- textDidEnd:textObject endChar:(unsigned short)whyEnd
{
    if ((whyEnd == 16) || (whyEnd == 17)) 
		[[swapView window] setDocEdited:NO];	// indicate change is saved

    return self;
}

@end

